/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */
package org.apache.qpid.server.security.access.plugins;


import org.apache.qpid.server.model.AccessControlProvider;
import org.apache.qpid.server.model.ManagedAttribute;
import org.apache.qpid.server.model.ManagedObject;
import org.apache.qpid.server.model.ManagedOperation;

@ManagedObject( category = false, type=AclFileAccessControlProvider.ACL_FILE_PROVIDER_TYPE )
public interface AclFileAccessControlProvider<X extends AclFileAccessControlProvider<X>> extends AccessControlProvider<X>
{
    String ACL_FILE_PROVIDER_TYPE = "AclFile";
    String PATH = "path";

    @ManagedAttribute( mandatory = true, description = "File location", oversize = true, oversizedAltText = OVER_SIZED_ATTRIBUTE_ALTERNATIVE_TEXT)
    String getPath();

    @ManagedOperation( description = "Causes the ACL rules to be reloaded.  Changes are applied immediately.",
            changesConfiguredObjectState = true)
    void reload();
}
