/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */
package org.apache.qpid.server.query.engine.retriever;

import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

import org.apache.qpid.server.model.ConfiguredObject;

/**
 * Parent interface for classes retrieving entities not available directly in broker hierarchy
 *
 * @param <C> Descendant of ConfiguredObject
 */
public interface EntityRetriever<C extends ConfiguredObject<?>>
{
    /**
     * Retrieves stream of entities
     *
     * @param broker Broker instance
     *
     * @return Stream of entities in form of key/value maps
     */
    Stream<Map<String, ?>> retrieve(final C broker);

    /**
     * Returns list of entity field names
     *
     * @return List of field names
     */
    List<String> getFieldNames();
}
